@@ -1,6 +1,6 @@
-# $Id: TLPSRC.pm 63068 2022-04-18 05:58:07Z preining $
+# $Id: TLPSRC.pm 67326 2023-06-11 15:34:16Z karl $
 # TeXLive::TLPSRC.pm - module for handling tlpsrc files
-# Copyright 2007-2021 Norbert Preining
+# Copyright 2007-2023 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
@@ -14,7 +14,7 @@ use TeXLive::TLUtils;
 use TeXLive::TLPOBJ;
 use TeXLive::TLTREE;
 
-my $svnrev = '$Revision: 63068 $';
+my $svnrev = '$Revision: 67326 $';
 my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
 sub module_revision { return $_modulerevision; }
 
@@ -113,6 +113,14 @@ sub from_file {
   for my $line (@lines) {
     $lineno++;
     
+    # remove end of line comments
+    # we require "<space>#" because since we don't want an embedded # in
+    # long descriptions, as in urls, to be a comment.
+    # Do this *before* we check for continuation lines: A line
+    #      .... # foobar \
+    # should *not* be treated as continuation line.
+    $line =~ s/\s+#.*$//;
+
     # we allow continuation lines in tlpsrc files, i.e., lines ending with \.
     if ($line =~ /^(.*)\\$/) {
       $savedline .= $1;
@@ -131,7 +139,10 @@ sub from_file {
     $line =~ /^ /
       && die "$srcfile:$lineno: non-continuation indentation not allowed: `$line'";
     #
-    # remove trailing white space.
+    # remove trailing comment (whitespace preceding #).
+    $line =~ s/\s+#.*$//;
+    #
+    # remove other trailing white space.
     $line =~ s/\s+$//;
 
     # expand tlpvars while reading in (except in descriptions).
@@ -145,12 +156,12 @@ sub from_file {
       # check that no variables remain unexpanded, or rather, for any
       # remaining $ (which we don't otherwise allow in tlpsrc files, so
       # should never occur) ... except for ${ARCH} which we specially
-      # expand in TLTREE.pm, and ${global_*} (and ${wndws}) which need to
+      # expand in TLTREE.pm, and ${global_*} which need to
       # be defined in 00texlive.autopatterns.tlpsrc. (We distribute one
       # file dvi$pdf.bat, but fortunately it is matched by a directory.)
       # 
       (my $testline = $line) =~ s,\$\{ARCH\},,g;
-      $testline =~ s,\$\{(global_[^}]*|wndws)\},,g;
+      $testline =~ s,\$\{(global_[^}]*)\},,g;
       $testline =~ /\$/
         && die "$srcfile:$lineno: variable undefined or syntax error: $line\n";
       #debug: warn "new line $line, from $origline\n" if $origline ne $line;
@@ -158,11 +169,11 @@ sub from_file {
 
     # names of source packages can either be
     # - normal names: ^[-\w]+$
-    # - win32 specific packages: ^[-\w]+\.win32$
+    # - windows specific packages: ^[-\w]+\.windows$
     # - normal texlive specific packages: ^texlive.*\..*$
     # - configuration texlive specific packages: ^00texlive.*\..*$
     if ($line =~ /^name\s/) {
-      $line =~ /^name\s+([-\w]+(\.win32)?|(00)?texlive\..*)$/;
+      $line =~ /^name\s+([-\w]+(\.windows)?|(00)?texlive\..*)$/;
       $foundnametag 
         && die "$srcfile:$lineno: second name directive not allowed: $line"
                . "(have $name)\n";
@@ -215,8 +226,8 @@ sub from_file {
     } elsif ($line =~ /^execute\s+(.*)$/) {
       push (@executes, $1) if ($1 ne "");
 
-    } elsif ($line =~ /^depend\s+(.*)$/) {
-      push (@depends, $1) if ($1 ne "");
+    } elsif ($line =~ /^(depend|hard)\s+(.*)$/) {
+      push (@depends, $2) if ($2 ne "");
 
     } elsif ($line =~ /^postaction\s+(.*)$/) {
       push (@postactions, $1) if ($1 ne "");
@@ -489,18 +500,18 @@ sub make_tlpobj {
       $finalp = "$pt $pr";
     }
     # one final trick
-    # if the original pattern string matches bin/win32/ then we *only*
-    # work on the win32 arch
-    if ($finalp =~ m! bin/win32/!) {
-      @todoarchs = qw/win32/;
+    # if the original pattern string matches bin/windows/ then we *only*
+    # work on the windows arch
+    if ($finalp =~ m! bin/windows/!) {
+      @todoarchs = qw/windows/;
     }
     # now @todoarchs contains only those archs for which we want
     # to match the pattern
-    foreach my $arch (@todoarchs) {
+    foreach my $arch (sort @todoarchs) {
       # get only those files matching the pattern
       my @archfiles = $tltree->get_matching_files('bin',$finalp, $pkgname, $arch);
       if (!@archfiles) {
-        if (($arch ne "win32") || defined($::tlpsrc_pattern_warn_win)) {
+        if (($arch ne "windows") || defined($::tlpsrc_pattern_warn_win)) {
           tlwarn("$self->{name} ($arch): no hit on binpattern $finalp\n");
         }
       }
@@ -533,11 +544,11 @@ sub make_tlpobj {
     }
     # now @todoarchs contains only those archs for which we want
     # to match the pattern
-    foreach my $arch (@todoarchs) {
+    foreach my $arch (sort @todoarchs) {
       # get only those files matching the pattern
       my @archfiles = $tltree->get_matching_files('bin', $finalp, $pkgname, $arch);
       if (!@archfiles) {
-        if (($arch ne "win32") || defined($::tlpsrc_pattern_warn_win)) {
+        if (($arch ne "windows") || defined($::tlpsrc_pattern_warn_win)) {
           tlwarn("$self->{name} ($arch): no hit on negative binpattern $finalp\n")
             unless $::tlpsrc_pattern_no_warn_negative;
             # see comments in libexec/place script.
@@ -585,7 +596,7 @@ sub _do_normal_pattern {
 # 
 # The returned hash has an additional key C<tlpvars> for global tlpsrc
 # variables, which can be used in any C<.tlpsrc> files. The names of these
-# variables all start with C<global_>, except for super-special ${wndws}.
+# variables all start with C<global_>.
 # 
 # =cut 
 # (all doc at bottom, let's not rewrite now.)
@@ -644,7 +655,7 @@ sub find_default_patterns {
   # check defined variables to ensure their names start with "global_".
   my %gvars = %{$tlsrc->_tlpvars};
   for my $v (keys %gvars) {
-    if ($v !~ /^(global_[-_a-zA-Z0-9]+|wndws)$/) {
+    if ($v !~ /^(global_[-_a-zA-Z0-9]+)$/) {
       tlwarn("$apfile: variable does not start with global_: $v\n")
         unless $v eq "PKGNAME";
         # the auto-defined PKGNAME is not expected to be global.
@@ -776,6 +787,7 @@ they are concatenated into C<foobar>.  The backslash and the newline are
 removed; no other whitespace is added or removed.
 
 Comment lines begin with a # and continue to the end of the line.
+Within a line, a # that is preceded by whitespace is also a comment.
 
 Blank lines are ignored.
 
@@ -900,7 +912,10 @@ For example, C<latex.tlpsrc> contains (among others):
   depend latex-fonts
   depend pdftex
 
-to ensure these packages are installed if the C<latex> package is.
+to ensure these packages are installed if the C<latex> package is.  The
+directive C<hard> is an alias for C<depend>, since that's we specified
+for the C<DEPENDS.txt> files package authors can provide; see
+L<https://www.tug.org/texlive/pkgcontrib.html#deps>.
 
 =head2 C<execute>
 
@@ -1004,7 +1019,7 @@ definition and expansion.)
 Ordinarily, variables can be used only within the C<.tlpsrc> file where
 they are defined. There is one exception: global tlpsrc variables can be
 defined in the C<00texlive.autopatterns.tlpsrc> file (mentioned below);
-their names must start with C<global_> (plus super-special C<wndws>),
+their names must start with C<global_>,
 and can only be used in C<depend>, C<execute>, and C<...pattern>
 directives, another C<tlpsetvar>. For example, our
 C<autopatterns.tlpsrc> defines:
@@ -1187,7 +1202,7 @@ all available architectures.
 
 =item C<bat/exe/dll/texlua> for Windows
 
-C<binpattern>s that match Windows, e.g., C<f bin/win32/foobar> or C<f
+C<binpattern>s that match Windows, e.g., C<f bin/windows/foobar> or C<f
 bin/${ARCH}/foobar>, also match the files C<foobar.bat>, C<foobar.cmd>,
 C<foobar.dll>, C<foobar.exe>, and C<foobar.texlua>.
 
@@ -1198,7 +1213,7 @@ architectures in one binpattern
 
   binpattern f bin/${ARCH}/dvips
 
-and would get C<bin/win32/dvips.exe> into the runfiles for C<arch=win32>.
+and would get C<bin/windows/dvips.exe> into the runfiles for C<arch=windows>.
 
 This C<bat>/C<exe>/etc. expansion I<only> works for patterns of the C<f>
 type.
@@ -1210,10 +1225,10 @@ package only for some architectures, or for all but some architectures.
 This can be done by specifying the list of architectures for which this
 pattern should be matched after the pattern specifier using a C</>:
 
-  binpattern f/win32 tlpkg/bin/perl.exe
+  binpattern f/windows tlpkg/bin/perl.exe
 
 will include the file C<tlpkg/bin/perl.exe> only in the binfiles for
-the architecture C<win32>. Another example:
+the architecture C<windows>. Another example:
 
   binpattern f/arch1,arch2,arch3 path/$ARCH/foo/bar
 
